"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _moment = _interopRequireDefault(require("moment"));

var _generatePassword = _interopRequireDefault(require("generate-password"));

var _bcrypt = require("../../lib/bcrypt");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const isVerifiedQuery = {
  password: {
    $exists: true
  },
  verified: true,
  active: true
};

class UserAuthDecorator {
  get isBlocked() {
    return 5 - this.loginAttempts <= 0;
  }

  get isPermitted() {
    return this.active && this.role && !this.frozen && !this.isBlocked;
  }

  get hasExpired() {
    const updatedAt = (0, _moment.default)().diff(this.secretIssuedOn);

    const diff = _moment.default.duration(updatedAt);

    return diff.asHours() > 24;
  }

  static async findByApiKey(str = '') {
    if (!str) return null;
    return this.findOne({
      apiKeys: str.slice(6, str.length).trim(),
      ...isVerifiedQuery
    });
  }

  static async $findOneStrictly(args) {
    const doc = await this.findOne(args);
    if (!doc) throw new Error('User not found');
    return doc;
  }

  static async findByEmail(email) {
    return this.$findOneStrictly({
      active: true,
      email
    });
  }

  static async findUnverifiedByEmail(email) {
    return this.$findOneStrictly({
      verified: false,
      active: true,
      email
    });
  }

  static async findUserBySecret(id, secret) {
    return this.$findOneStrictly({
      active: true,
      _id: id,
      secret
    });
  }

  static async findVerifiedByEmail(email) {
    return this.$findOneStrictly({ ...isVerifiedQuery,
      email
    });
  }

  static async findVerifiedById(id) {
    return this.$findOneStrictly({ ...isVerifiedQuery,
      _id: id
    });
  }

  async setSecret() {
    this.secret = (0, _bcrypt.generateRandomSecret)();
    this.secretIssuedOn = new Date();
    await this.save();
  }

  async setPassword(s) {
    const re = /^(?=.*[a-z])(?=.*[A-Z])(?=.*[0-9])(?=.*[!@#$%^&*])(?=.{8,})/;

    if (s && !re.test(s)) {
      throw new Error('Password too weak');
    }

    const password = s || _generatePassword.default.generate({
      length: 10,
      numbers: true,
      symbols: true,
      uppercase: true
    });

    this.set({
      password: await (0, _bcrypt.createHash)(password),
      loginAttempts: 0,
      verified: true
    });
    await this.save();
    return password;
  }

  async verifyPassword(password, strict = false) {
    const matches = (0, _bcrypt.compareWithHash)(password, this.password);
    this.isNew = false;
    this.loginAttempts = !matches ? Number(this.loginAttempts || 0) + 1 : 0;
    await this.save();

    if (!matches && strict) {
      throw new Error('Incorrect password');
    }

    return matches;
  }

  async deactivate() {
    this.set({
      verified: false,
      active: false,
      secret: null,
      password: null
    });
    await this.save();
  }

  obfuscatePrivateFields() {
    const obj = this.toJSON();
    delete obj.password;
    delete obj.secret;
    obj.apiKeys = obj.apiKeys.map(v => {
      let masked = '';
      const maskedLen = v.length - 4;

      for (let i = 0; i < maskedLen; i += 1) {
        masked += '*';
      }

      return masked + v.slice(maskedLen);
    });
    return obj;
  }

  async generateApiKey() {
    const key = (0, _bcrypt.generateRandomSecret)();
    const stringify = `${String(this._id).substring(0, 7)}-${key}-${(0, _moment.default)(new Date()).unix()}`;
    this.apiKeys.push(stringify);
    await this.save();
    return stringify;
  }

}

exports.default = UserAuthDecorator;