"use strict";

var _express = _interopRequireDefault(require("express"));

var _mongoose = _interopRequireDefault(require("mongoose"));

var _moment = _interopRequireDefault(require("moment"));

var _bodyParser = _interopRequireDefault(require("body-parser"));

var _supertest = _interopRequireDefault(require("supertest"));

var _access = _interopRequireDefault(require("../../models/access"));

var _user = _interopRequireDefault(require("../../models/user"));

var _account = _interopRequireDefault(require("../account"));

var _jwt = require("../../lib/jwt");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let id;
let secret;
let User;
const app = (0, _express.default)();
const request = (0, _supertest.default)(app);
const email = 'morty@adult.swim';
const password = 'Let%09823m807!*8';

const setPassword = async () => {
  const doc = await User.findById(id);
  await doc.setPassword(password);
};

beforeAll(async () => {
  process.env.SECRET = 'HEY';
  User = _mongoose.default.model('mock-user', _user.default);

  const Access = _mongoose.default.model('mock-access', _access.default);

  const doc = await User.create({
    firstName: 'Jon',
    lastName: 'Doe',
    secret: 'secret',
    active: true,
    role: 'Supert',
    email
  });
  ({
    _id: id,
    secret
  } = doc);
  app.use(_bodyParser.default.json());
  app.use((req, res, next) => {
    req.user = doc;
    next();
  });
  app.use((0, _account.default)(_mongoose.default, {
    Access,
    User
  }));
});
describe('/verify', () => {
  it('should return 500', async () => {
    await request.post('/verify').expect(500);
  });
  it('should return 422', async () => {
    await request.post('/verify').send({
      id,
      verificationCode: secret
    }).expect(422);
  });
  it('should return 204', async () => {
    await request.post('/verify').send({
      id,
      verificationCode: secret,
      password: 'Thi5!sSt-ng!2'
    }).expect(204);
  });
  it('should return 409', async () => {
    await User.findByIdAndUpdate(id, {
      secretIssuedOn: (0, _moment.default)().subtract(2, 'weeks')
    });
    await request.post('/verify').send({
      id,
      verificationCode: secret
    }).expect(410);
  });
});
describe('/authenticate', () => {
  const performAuthentication = () => request.post('/authenticate').send({
    password,
    email
  });

  beforeAll(async () => {
    await setPassword();
  });
  it('should return 201', async () => {
    const {
      body
    } = await performAuthentication().expect(201);
    expect(body).toMatchObject({
      token: expect.any(String),
      nonce: expect.any(String)
    });
  });
  it('should return 403', async () => {
    await User.findByIdAndUpdate(id, {
      loginAttempts: 12
    });
    await performAuthentication().expect(403);
  });
  it('should return 401', async () => {
    const doc = await User.findById(id);
    await doc.setPassword();
    await performAuthentication().expect(401);
    expect((await User.findById(id).select('loginAttempts').lean())).toHaveProperty('loginAttempts', 1);
  });
});
describe('/reverify', () => {
  const requestReverification = () => request.post('/reverify').send({
    email
  });

  beforeAll(async () => {
    await User.findByIdAndUpdate(id, {
      verified: false
    });
  });
  it('should return 204', async () => {
    await requestReverification().expect(204);
  });
  it('should return 500', async () => {
    await User.findByIdAndUpdate(id, {
      verified: true
    });
    await requestReverification().expect(500);
  });
});
describe('/password-*', () => {
  beforeAll(async () => {
    await setPassword();
  });
  it('should return 204', async () => {
    await request.post('/password-update').send({
      previousPassword: password,
      newPassword: 'newPas9087324u9*9%*^',
      confirmNewPassword: 'newPas9087324u9*9%*^'
    }).expect(204);
  });
  it('should return 422', async () => {
    await request.post('/password-update').send({
      previousPassword: password,
      newPassword: 'newPas90f9*9%*^',
      confirmNewPassword: 'newPas9087324u9*9%*^'
    }).expect(422);
  });
  it('should return 204', async () => {
    await request.post('/password-reset').send({
      email
    }).expect(204);
  });
});
describe('/profile', () => {
  beforeAll(async () => {
    await setPassword();
  });
  it('should return 200', async () => {
    const {
      token,
      nonce
    } = await (0, _jwt.generateIDToken)(id, secret, 'localhost');
    const {
      body
    } = await request.get('/profile').set('Authorization', `Bearer ${token}`).set('exp-nonce', nonce).set('host', 'localhost').expect(200);
    expect(body).toHaveProperty('profile');
    expect(body).toHaveProperty('permissions');
  });
});